package dk.nsp.idsas.integration.personinformation;

import dk.nsp.idsas.integration.ServiceErrorCallback;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.time.Duration;
import java.util.Map;

public class PersonInformationClient {

    private static final String APPLICATION_JSON = "application/json";

    private static final String CONTENT_TYPE = "Content-Type";

    private static final Logger LOGGER = Logger.getLogger(PersonInformationClient.class);

    private final ObjectMapper objectMapper = new ObjectMapper();

    private final ServiceErrorCallback serviceErrorCallback;

    private final HttpClient httpClient;

    private final String url;

    public PersonInformationClient(HttpClient httpClient, String url, ServiceErrorCallback serviceErrorCallback) {
        this.httpClient = httpClient;
        this.url = url;
        this.serviceErrorCallback = serviceErrorCallback;
    }

    public PersonInformationClient(HttpClient httpClient, String url, Duration errorCountDuration) {
        this(httpClient, url, new ServiceErrorCallback(errorCountDuration));
    }

    public boolean personExists(String cpr) throws PersonInformationClientException {
        HttpUriRequest cprRequest = buildGetPersonInformationRequest(cpr);
        return invokeService(cprRequest);
    }

    public int getErrorCount() {
        return serviceErrorCallback.getErrorCount();
    }

    private HttpUriRequest buildGetPersonInformationRequest(String cpr) {
        // Example request string: http://localhost:8086/stamdata-personinformation/person/1212121212
        return new HttpGet(String.format("%s/person/%s", url, cpr));
    }

    private boolean invokeService(HttpUriRequest request) throws PersonInformationClientException {
        try {
            HttpResponse response = httpClient.execute(request);
            boolean result = handleResponse(response);
            serviceErrorCallback.registerServiceSuccess();
            return result;
        } catch (IOException e) {
            String errorMessage = "CPR-nummeret kunne ikke valideres, fordi kaldet til PersonInformation-servicen gik galt.";
            LOGGER.error(errorMessage, e);
            serviceErrorCallback.registerServiceError();
            throw new PersonInformationClientException(errorMessage, e);
        } catch (PersonInformationClientException e) {
            serviceErrorCallback.registerServiceError();
            throw e;
        }
    }

    private boolean handleResponse(HttpResponse response) throws PersonInformationClientException {
        int statusCode = response.getStatusLine().getStatusCode();
        switch (statusCode) {
            case HttpStatus.SC_OK: {
                boolean contentTypeOk = contentTypeOk(response);
                Map<String, Object> responseJson = parseResponseAsJson(response);
                if (!contentTypeOk || !responseJson.containsKey("cpr")) {
                    LOGGER.error("Fejl i retursvar fra PersonInformation (200)");
                    throw new PersonInformationClientException("Fejl i retursvar fra PersonInformation");
                }
                return true;
            }
            case HttpStatus.SC_NOT_FOUND: {
                boolean contentTypeOk = contentTypeOk(response);
                if (!contentTypeOk) {
                    LOGGER.error("Fejl i retursvar fra PersonInformation (404) forkert contenttype");
                    throw new PersonInformationClientException("Fejl i retursvar fra PersonInformation");
                }
                parseResponseAsJson(response);
                return false;
            }
            default:
                throw new PersonInformationClientException(String.format("CPR-nummeret kunne ikke valideres, fordi kaldet til PersonInformation-servicen returnerede en uventet statuskode: %d", statusCode));
        }
    }

    private Map<String, Object> parseResponseAsJson(HttpResponse response) throws PersonInformationClientException {
        try {
            return objectMapper.readValue(response.getEntity().getContent(), Map.class);
        } catch (UnsupportedOperationException | IOException e) {
            LOGGER.error("Kunne ikke parse response body til json", e);
            throw new PersonInformationClientException("Fejl i fortolkning af svar fra PersonInformation", e);
        }
    }

    private boolean contentTypeOk(HttpResponse response) {
        Header headers = response.getFirstHeader(CONTENT_TYPE);
        return headers != null && headers.getValue().equalsIgnoreCase(APPLICATION_JSON);
    }

}
