package dk.nsp.idsas.integration.sores;

import dk.nsp.idsas.integration.ServiceErrorCallback;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

public class SoresClient {
    private static final String CONTENT_TYPE = "application/json";
    private static final Logger logger = Logger.getLogger(SoresClient.class);

    private final ObjectMapper objectMapper = new ObjectMapper();
    private final String url;
    private final HttpClient httpClient;
    private final ServiceErrorCallback serviceErrorCallback;

    public static final String SORES_ERROR = "CVR-nummeret kunne ikke valideres, fordi kaldet til SORES gik galt.";
    public static final String SORES_RESPONSE_ERROR = "Svaret fra SORES havde et uventet format.";

    public SoresClient(HttpClient httpClient, String url, ServiceErrorCallback serviceErrorCallback) {
        this.url = url;
        this.httpClient = httpClient;
        this.serviceErrorCallback = serviceErrorCallback;
    }

    public int getErrorCount() {
        return serviceErrorCallback.getErrorCount();
    }

    public boolean cvrMapsToSor(String cpr) throws SoresClientException {
        try {
            HttpUriRequest request = buildPostRequest(cpr);
            return invokeService(request);
        } catch (JsonProcessingException | UnsupportedOperationException | UnsupportedEncodingException e) {
            throw new SoresClientException(e.getMessage(), e);
        }
    }

    private HttpUriRequest buildPostRequest(String inputCvr) throws JsonProcessingException, UnsupportedEncodingException {
        final HttpPost httpPost = new HttpPost(url);
        final Object requestBody = new Object() {
            public final String op = "EntityNameLookup";
            public final String matchEntityName = "";
            public final String cvr = inputCvr;
        };

        // using Jackson to serialize to JSON
        final ObjectMapper objectMapper = new ObjectMapper();
        final String jsonString = objectMapper.writeValueAsString(requestBody);
        final StringEntity entity = new StringEntity(jsonString);
        httpPost.setEntity(entity);
        httpPost.setHeader("Content-type", CONTENT_TYPE);
        return httpPost;
    }

    private boolean invokeService(HttpUriRequest request) throws SoresClientException {
        try {
            HttpResponse response = httpClient.execute(request);
            return handleResponse(response);
        } catch (IOException e) {
            serviceErrorCallback.registerServiceError();
            throw new SoresClientException(SORES_ERROR, e);
        }
    }

    private boolean handleResponse(HttpResponse response) throws SoresClientException, IOException {
        JsonNode jsonNode = objectMapper.readTree(response.getEntity().getContent());

        if (response.getStatusLine().getStatusCode() == HttpStatus.SC_NOT_FOUND) {
            serviceErrorCallback.registerServiceSuccess();
            return false;
        }

        if (response.getStatusLine().getStatusCode() == HttpStatus.SC_BAD_REQUEST) {
            serviceErrorCallback.registerServiceError();
            throw new SoresClientException(SORES_ERROR);
        }

        if (jsonNode.get("") == null) {
            serviceErrorCallback.registerServiceError();
            throw new SoresClientException(SORES_RESPONSE_ERROR);
        }

        int responseSize = jsonNode.get("").size();
        if (responseSize > 0) {
            logger.debug("SOR codes returned: " + responseSize);
            serviceErrorCallback.registerServiceSuccess();
            return true;
        }

        return false;
    }

}
