package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.integration.personinformation.PersonInformationClient;
import dk.nsp.idsas.integration.personinformation.PersonInformationClientException;
import dk.nsp.idsas.integration.sores.SoresClient;
import dk.nsp.idsas.integration.sores.SoresClientException;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;

import java.util.Calendar;
import java.util.GregorianCalendar;

public class BlurringService {
    private WhitelistDao whitelistDao;
    private BlurringDao blurringDao;
    private AuditLogger auditLogger;
    private PersonInformationClient personInformationClient;
    private SoresClient soresClient;

    public BlurringService(WhitelistDao whitelistDao, BlurringDao blurringDao, AuditLogger auditLogger, PersonInformationClient personInformationClient, SoresClient soresClient) {
        this.whitelistDao = whitelistDao;
        this.blurringDao = blurringDao;
        this.auditLogger = auditLogger;
        this.personInformationClient = personInformationClient;
        this.soresClient = soresClient;
    }

    public void createBlurring(IdsasActor actor, String idClassification, String cpr, GregorianCalendar endDateTime) throws IdsasServiceException {

        // Tjek at CVR er whitelisted til at kalde createBlurring
        boolean isWhiteListed = whitelistDao.isWhitelisted(actor.getCvr(), WhitelistType.BLURRING);
        if (!isWhiteListed) {
            throw new IdsasServiceException(ErrorMessages.cvrWhitelistError(actor.getCvr(), WhitelistType.BLURRING), true);
        }

        // Tjek at CVR mapper til en SOR kode (via SORES)
        try {
            boolean cvrMapsToSor = soresClient.cvrMapsToSor(actor.getCvr());
            if (!cvrMapsToSor) {
                throw new IdsasServiceException(ErrorMessages.cvrDoesNotMapToSorError(actor.getCvr()), true);
            }
        } catch (SoresClientException e) {
            throw new IdsasServiceException(e.getMessage(), false);
        }

        // Tjek at patientId er gyldig (via "personinformation")
        boolean patientIdIsValid = false;
        try {
            patientIdIsValid = personInformationClient.personExists(cpr);
            if (!patientIdIsValid) {
                throw new IdsasServiceException(ErrorMessages.cprIsNotValidError(cpr), true);
            }
        } catch (PersonInformationClientException e) {
            throw new IdsasServiceException(e.getMessage(), false);
        }

        // Tjek at sløringen ikke ligger mere end 2 år fremme i tiden
        Calendar maxDateTime = new GregorianCalendar();
        maxDateTime.add(Calendar.YEAR, 2);
        if (endDateTime.after(maxDateTime)) {
            throw new IdsasServiceException(ErrorMessages.maxExpiryDateExceeded(), true);
        }

        // Entity oprettes og gemmes
        BlurringEntity entity = new BlurringEntity(
                cpr,
                idClassification,
                actor.getCvr(),
                "CVR",
                endDateTime.toZonedDateTime().toInstant());

        blurringDao.insert(entity);

        // Audit logning
        auditLogger.logCreateBlurring(cpr, actor.getCvr());
    }
}
