package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.integration.personinformation.PersonInformationClient;
import dk.nsp.idsas.integration.personinformation.PersonInformationClientException;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.integration.sores.SoresClient;
import dk.nsp.idsas.integration.sores.SoresClientException;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Date;
import java.util.GregorianCalendar;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

public class BlurringServiceTest {
    private WhitelistDao whitelistDao;
    private BlurringDao blurringDao;
    private AuditLogger auditLogger;
    private PersonInformationClient personInformationClient;
    private BlurringService blurringService;
    private SoresClient soresClient;

    @Before
    public void setUp() {
        whitelistDao = Mockito.mock(WhitelistDao.class);
        blurringDao = Mockito.mock(BlurringDao.class);
        auditLogger = Mockito.mock(AuditLogger.class);
        personInformationClient = Mockito.mock(PersonInformationClient.class);
        soresClient = Mockito.mock(SoresClient.class);
        blurringService = new BlurringService(whitelistDao, blurringDao, auditLogger, personInformationClient, soresClient);
    }

    @Test
    public void createBlurring_WhenCvrIsNotWhitelisted_ThrowsException() {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "TestSystem", "");
        Mockito.when(whitelistDao.isWhitelisted(actor.getCvr(), WhitelistType.BLURRING)).thenReturn(false);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor, "", "", toGregorian(LocalDateTime.now())));

        // Assert
        assertEquals(ErrorMessages.cvrWhitelistError(actor.getCvr(), WhitelistType.BLURRING), e.getMessage());
    }

    @Test
    public void createBlurring_WhenCvrDoesNotMapToSor_ThrowsException() throws SoresClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "TestSystem", "");
        Mockito.when(whitelistDao.isWhitelisted(actor.getCvr(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(soresClient.cvrMapsToSor(actor.getCvr())).thenReturn(false);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor, "", "", toGregorian(LocalDateTime.now())));

        // Assert
        assertEquals(ErrorMessages.cvrDoesNotMapToSorError(actor.getCvr()), e.getMessage());
    }

    @Test
    public void createBlurring_WhenNoPersonInformationIsFoundForCpr_ThrowsException() throws PersonInformationClientException, SoresClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "TestSystem", "");
        String cpr = "1111111111";
        Mockito.when(whitelistDao.isWhitelisted(actor.getCvr(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(soresClient.cvrMapsToSor(actor.getCvr())).thenReturn(true);
        Mockito.when(personInformationClient.personExists(cpr)).thenReturn(false);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor, "", cpr, toGregorian(LocalDateTime.now())));

        // Assert
        assertEquals(ErrorMessages.cprIsNotValidError(cpr), e.getMessage());
    }

    @Test
    public void createBlurring_WhenAllValidationsSucceed_CreatesBlurringAndAuditLogs() throws PersonInformationClientException, IdsasServiceException, SoresClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "TestSystem", "");
        String idClassification = "testClassification";
        String cpr = "1111111111";
        GregorianCalendar endDateTime = toGregorian(LocalDateTime.now());
        Mockito.when(whitelistDao.isWhitelisted(actor.getCvr(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(soresClient.cvrMapsToSor(actor.getCvr())).thenReturn(true);
        Mockito.when(personInformationClient.personExists(cpr)).thenReturn(true);

        // Act
        blurringService.createBlurring(actor, idClassification, cpr, endDateTime);

        // Assert
        Mockito.verify(blurringDao).insert(Mockito.argThat(blurring -> {
            assertEquals(cpr, blurring.getPatientId());
            assertEquals(idClassification, blurring.getPatientIdType());
            assertEquals(actor.getCvr(), blurring.getOrganisationId());
            assertEquals("CVR", blurring.getOrganisationIdType());
            assertEquals(endDateTime.toInstant(), blurring.getExpiryDate());
            return true;
        }));
        Mockito.verify(auditLogger).logCreateBlurring(cpr, actor.getCvr());
    }

    @Test
    public void createBlurring_WhenExpiryDateIsExceeded_ThrowsException() throws PersonInformationClientException, SoresClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "TestSystem", "");
        GregorianCalendar exceededDate =  toGregorian(LocalDateTime.now().plusYears(2).plusDays(1));
        String idClassification = "testClassification";
        String cpr = "1111111111";
        Mockito.when(whitelistDao.isWhitelisted(actor.getCvr(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(soresClient.cvrMapsToSor(actor.getCvr())).thenReturn(true);
        Mockito.when(personInformationClient.personExists(cpr)).thenReturn(true);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor, idClassification, cpr, exceededDate));

        // Assert
        assertEquals(ErrorMessages.maxExpiryDateExceeded(), e.getMessage());
    }

    private GregorianCalendar toGregorian(LocalDateTime datetime) {
        GregorianCalendar gregorianCalendar = (GregorianCalendar) GregorianCalendar.getInstance();
        gregorianCalendar.setTime(Date.from(datetime.toInstant(ZoneOffset.UTC)));
        return gregorianCalendar;
    }

}
